#include "Pledge.h"
#include "sys/Syscall.h"
#include "thread/Scheduler.h"
#include "thread/Timer.h"
#include <luna/Common.h>

Result<u64> sys_alarm(Registers*, SyscallArgs args)
{
    unsigned int seconds = (unsigned int)args[0];

    auto* current = Scheduler::current();

    TRY(check_pledge(current, Promise::p_stdio));

    Clock* clock = nullptr;

    u64 ticks_left = current->timer ? current->timer->clock->ticks_left(current->timer) : 0;

    if (current->timer)
    {
        clock = current->timer->clock;
        if (clock) clock->remove_from_timer_queue(current->timer);
    }
    else
        current->timer = TRY(make<Timer>());

    if (!clock) clock = &g_realtime_clock;

    current->timer->total_ticks = ceil_div((static_cast<long>(seconds) * 1'000'000'000), clock->resolution());
    current->timer->thread = current;

    clock->add_to_timer_queue(current->timer);

    return (ticks_left * clock->resolution()) / 1'000'000'000;
}
